package hikapro.com.backpack.model;


import android.os.Message;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

import hikapro.com.backpack.model.dao.Command;
import hikapro.com.backpack.model.dao.DAO;
import hikapro.com.backpack.model.dao.Event;
import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.presenter.Presenter;

/**
 * Created by tariel on 22/04/16.
 */
public class ItemModel implements Model.Item {

    protected Presenter.ItemList  presenter;
    private List<Category>      rawCategories;
    private List<Category>      sortedCategories;
    private List<Item>          rawItems;

    protected DAO               dao;
    protected int               currentSet;
    protected int               currentSetActiveItemsQty;
    protected int               packedQty;
    protected Hashtable<Integer, Category>      categoriesCache;
    private List<Item>          itemsCache;
    private List<Item>          itemsDiscardCache;

    private Hashtable<Category, List<Item>> items;
    protected Hashtable<Integer, List<Item>> cache;

    public ItemModel() {
        this.rawCategories      = new ArrayList<>();
        this.rawItems           = new ArrayList<>();
        this.sortedCategories   = new ArrayList<>();

        this.categoriesCache    = new Hashtable<>(20, 0.9f);

        this.packedQty          = -1;

        this.itemsCache         = new ArrayList<>();
        this.itemsDiscardCache  = new ArrayList<>();
        this.cache              = new Hashtable<>(12, 0.9f);
        this.dao                = DAO.getInstance();
        dao.registerObserver(this);
    }

    // categories -->

    @Override
    public Category getCategoryByPosition(int position) {
        Category ret = null;
        if (cache.containsKey(currentSet))
            ret = categoriesCache.get(cache.get(currentSet).get(position).getCategory());
        return ret;

    }
    // categories <--

    // items -->

    @Override
    public int insertItem(Item item) {
        List<Item> iList = cache.get(currentSet);
        iList.add(item);
        Collections.sort(iList);
        cache.put(currentSet, iList);
        ++currentSetActiveItemsQty;
        // TODO write to database
        return 0;
    }
    @Override
    public boolean deleteItem(Item item) {
        if (isPendingRemoval(item))
            itemsDiscardCache.remove(item);
        cache.get(currentSet).remove(item);
        --currentSetActiveItemsQty;
        //TODO write to ds

        return false;
    }

    @Override
    public Item findItem(int id) {
        List<Item> items = cache.get(currentSet);
        Item item = null;
        if (items != null)
        {
            for (Item i : items) {
                if (i.getId() == id) {
                    item = i;
                    break;
                }
            }
        }
        return item;
    }
    @Override
    public Item getItemByPosition(int position) {
        Item ret = null;
        if (cache.containsKey(currentSet))
            ret = cache.get(currentSet).get(position);
        return ret;
    }


    @Override
    public void filter(String query) {

        if (query.isEmpty()) {
            Message command = Message.obtain();
            command.what = Command.SET_GET_ITEMS;
            command.arg1 = presenter.getCurrentSet().getId();
            dao.executeCommand(command);
        } else {
            query = query.toLowerCase();
            String name;
            List<Item> newList = new ArrayList<>(20);
            List<Item> oldList = cache.get(currentSet);
            for (Item item : oldList) {
                name = item.getName().toLowerCase();
                if (name.contains(query)) {
                    newList.add(item);
                }
            }
            cache.put(currentSet, newList);
        }
    }

    @Override
    public int getHeaderId(int position) {
        return cache.containsKey(currentSet) ? cache.get(currentSet).get(position).getCategory() : -1;
    }

    @Override
    public int getItemId(int position) {
        return cache.containsKey(currentSet) ? cache.get(currentSet).get(position).getId() : -1;
    }

    @Override
    public void clear() {
        if (cache.containsKey(currentSet))
            cache.get(currentSet).clear();
    }

    @Override
    public boolean isPendingRemoval(Item item) {
        return itemsDiscardCache.contains(item);
    }

    @Override
    public void pendingRemoveCancel(Item item) {
        if (itemsDiscardCache.contains(item))
            itemsDiscardCache.remove(item);
    }

    @Override
    public void pendingRemove(Item item) {
        itemsDiscardCache.add(item);
    }

    @Override
    public int getItemsCount() {
        boolean is = cache.containsKey(currentSet);
        return is ? cache.get(currentSet).size() : 0;
    }

    @Override
    public int getActiveItemsCount() {
        return currentSetActiveItemsQty;
    }

    @Override
    public int getPackedQty() {
        return packedQty;
    }
    // items <--

    // events -->

    @Override
    public void notifyDataSetChanged() {
        presenter.notifyDataSetChanged();
    }

    @Override
    public void onDestroy(boolean isConfigurationChanging) {
        if ( !isConfigurationChanging ) {
            presenter = null;
        }
    }

    private void sendMessage(String message) {
        presenter.showMessage(message);
    }

    // events <--

    // process -->


    @Override
    public void packItem(int itemId) {
        Item item = findItem(itemId);
        if (item != null)
            cache.get(currentSet).remove(item);
        Message command = Message.obtain();
        command.what = Command.ITEM_PACK;
        command.arg1 = currentSet;
        command.arg2 = itemId;
        dao.executeCommand(command);
    }

    @Override
    public void unpackItem(int itemId) {
        // nothing
    }

    @Override
    public void unpackSet(int setId) {
        // nothing
    }

    @Override
    public void executeQuery() {
        Message command;

        if (cache.contains(currentSet)) {
            notifyDataSetChanged();
        } else {
            if (categoriesCache.isEmpty()) {
                command = Message.obtain();
                command.what = Command.ITEM_GET_CATEGORIES;
                dao.executeCommand(command);
            }
            command = Message.obtain();
            command.what = Command.SET_GET_ITEMS;
            command.arg1 = presenter.getCurrentSet().getId();
            dao.executeCommand(command);
        }
    }
    @Override
    public void onEvent(Message event) {

        switch (event.what) {
            case Event.SET_ITEMS_LOAD_ERROR : // TODO check
                if (presenter != null) {
                    if (packedQty == -1)
                        packedQty = presenter.getCurrentSet().getPackedQty();
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.SET_UNPACK_ERROR :
                break;
            case Event.ITEM_FROM_SET_ERROR :
                break;
            case Event.ITEM_DELETE_ERROR :
                break;
            case Event.ITEM_PACK_ERROR :
                break;
            case Event.ITEM_UNPACK_ERROR :
                break;
            case Event.ITEM_INSERT_ERROR :
                break;
            case Event.ITEM_CATEGORY_LOAD_ERROR :
                break;
            case Event.SET_ITEMS_LOAD_COMPLETED :
                List<Item> res = (List<Item>) event.obj;
                cache.put(event.arg1, res);
                notifyDataSetChanged();
                if (presenter != null) {
                    if (packedQty == -1)
                        packedQty = presenter.getCurrentSet().getPackedQty();
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.SET_UNPACK_COMPLETED :
                packedQty = 0;
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.ITEM_CATEGORY_LOAD_COMPLETED :
                categoriesCache = (Hashtable<Integer, Category>)event.obj;
                break;
            case Event.ITEM_FROM_SET_DELETED :
                break;
            case Event.ITEM_DELETED :
                break;
            case Event.ITEM_PACKED :
                //TODO something
                ++packedQty;
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.ITEM_UNPACKED :
                --packedQty;
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.ITEM_INSERTED :
                break;
        }
    }

    // process <--

    // other -->


    @Override
    public void setPresenter(Presenter.ItemList presenter) {
        this.presenter      = presenter;
        this.currentSet     = presenter.getCurrentSet().getId();
        this.currentSetActiveItemsQty = presenter.getCurrentSet().getActiveQty();
    }

    @Override
    public Presenter.ItemList getPresenter() {
        return presenter;
    }

    private Category findCategory(int categoryId) {
        Category category = null;

        for (Category c : rawCategories) {
            if (c.getId() == categoryId) {
                category = c;
                break;
            }
        }
        return category;
    }
    // other <--
}
