package hikapro.com.backpack.model;

import android.os.Message;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;

import hikapro.com.backpack.model.dao.Command;
import hikapro.com.backpack.model.dao.DAO;
import hikapro.com.backpack.model.dao.Event;
import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.presenter.Presenter;

/**
 * Created by tariel on 12/05/16.
 */
public class PackedModel extends ItemModel {

    public PackedModel() {
        super();
    }

    @Override
    public void executeQuery() {
        Message command;

        if (cache.contains(currentSet)) {
            notifyDataSetChanged();
        } else {
            if (categoriesCache.isEmpty()) {
                command = Message.obtain();
                command.what = Command.ITEM_GET_CATEGORIES;
                dao.executeCommand(command);
            }
            command = Message.obtain();
            command.what = Command.SET_GET_PACKED_ITEMS;
            command.arg1 = presenter.getCurrentSet().getId();
            dao.executeCommand(command);
        }
    }

    @Override
    public void onEvent(Message event) {

        switch (event.what) {

            case Event.SET_PACKED_LOAD_ERROR :
                break;
            case Event.SET_UNPACK_ERROR :
                break;
            case Event.ITEM_CATEGORY_LOAD_ERROR :
                break;
            case Event.ITEM_PACK_ERROR :
                break;
            case Event.ITEM_UNPACK_ERROR :
                break;
            case Event.SET_PACKED_LOAD_COMPLETED :
                List<Item> res = (List<Item>) event.obj;
                cache.put(event.arg1, res);
                notifyDataSetChanged();
                packedQty = res.size();
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.SET_UNPACK_COMPLETED :
                packedQty = 0;
                cache.get(event.arg2).clear();
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.ITEM_CATEGORY_LOAD_COMPLETED :
                categoriesCache = (Hashtable<Integer, Category>)event.obj;
                break;
            case Event.ITEM_PACKED :
                //TODO something
                ++packedQty;
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
            case Event.ITEM_UNPACKED :
                --packedQty;
                if (presenter != null) {
                    presenter.notifyItemPackStatusChanged();
                }
                break;
        }
    }

    @Override
    public void unpackItem(int itemId) {
        Message command;
        command = Message.obtain();
        command.what = Command.ITEM_UNPACK;
        command.arg1 = currentSet;
        command.arg2 = itemId;
        dao.executeCommand(command);
        Item item = findItem(itemId);
        if (item != null)
            cache.get(currentSet).remove(item);
    }

    @Override
    public void packItem(int itemId) {
        // nothing
    }

    @Override
    public void unpackSet(int setId) {
        Message command;
        command = Message.obtain();
        command.what = Command.SET_UNPACK_ITEMS;
        command.arg1 = setId;
        dao.executeCommand(command);
    }
}
