package hikapro.com.backpack.model.dao;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;

import hikapro.com.backpack.App;

/**
 * Created by tariel on 04/05/16.
 */
public class ImageDownloadHelper {

    public static final String ITEMS_TAG = "items";
    private Context context;

    public ImageDownloadHelper() {
        this.context = App.getAppContext();
    }

    public Bitmap loadImage(String netPath) {
        Bitmap bitmap = null;
        try {
            URL url = new URL(netPath);
            URLConnection conn = url.openConnection();
            bitmap = BitmapFactory.decodeStream(conn.getInputStream());
        } catch (Exception ex) {
            Log.e(ITEMS_TAG, " File cannot be downloaded");
        }
        return bitmap;
    }

    public boolean existsExternal(String filename) {
        boolean ret = false;
        if (isExternalStorageReadable()) {
            File directory = getDir();
            File file = new File(directory.getAbsoluteFile(), filename);
            ret = file.exists();
        }
        return ret;
    }
    public boolean existsInternal(String filename) {
        boolean ret = false;
        File file = createTempFile(filename);
        if (file != null) {
            ret = file.exists();
        }
        return ret;
    }

    private File createTempFile(String fileName) {
        File file = null;
        try {
            file = File.createTempFile(fileName, null, context.getCacheDir());
        }  catch (IOException e) {
            Log.e(ITEMS_TAG, " Cannot obtain temp file");
        }
        return file;
    }

    public String generateFileName(String netPath) {
        String ret = "";
        if (netPath != null && !netPath.isEmpty()) {
            ret = Uri.parse(netPath).getLastPathSegment();
        }
        return ret;
    }

    public String saveImageInternal(String filename, Bitmap bitmap) {
        File file = null;
        FileOutputStream outputStream;
        if (bitmap != null && !filename.isEmpty()) {
            file = createTempFile(filename);
            if (file != null) {
                try {
                    outputStream = new FileOutputStream(file);
                    bitmap.compress(Bitmap.CompressFormat.JPEG, 90, outputStream);
                    outputStream.flush();
                    outputStream.close();
                } catch (Exception e) {
                    Log.e(ITEMS_TAG, " File cannot be saved");
                }
            }
        }
        return file != null ? file.getAbsolutePath() : null;
    }

    public String saveImageExternal(String filename, Bitmap bitmap) {
        File file = null;
        FileOutputStream outputStream;
        if (isExternalStorageWritable()) {
            if (bitmap != null && !filename.isEmpty()) {
                File directory = getDir();
                file = new File(directory.getAbsoluteFile(), filename);
                try {
                    outputStream = new FileOutputStream(file);
                    bitmap.compress(Bitmap.CompressFormat.JPEG, 90, outputStream);
                    outputStream.flush();
                    outputStream.close();
                } catch (Exception e) {
                    Log.e(ITEMS_TAG, " File cannot be saved");
                }
            }
        }
        return file != null ? file.getAbsolutePath() : null;
    }

    /* Checks if external storage is available for read and write */
    public boolean isExternalStorageWritable() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state)) {
            return true;
        }
        return false;
    }

    /* Checks if external storage is available to at least read */
    public boolean isExternalStorageReadable() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state) ||
                Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
            return true;
        }
        return false;
    }

    private File getDir() {
        // Get the directory for the app's private pictures directory.
        File file = new File(context.getExternalFilesDir(
                Environment.DIRECTORY_PICTURES), ITEMS_TAG);
        if (!file.mkdirs()) {
            Log.e(ITEMS_TAG, " Directory not created");
        }
        return file;
    }
}
