package hikapro.com.backpack.presenter;

import android.app.Fragment;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.adapters.PackedListAdapter;
import hikapro.com.backpack.presenter.adapters.helper.items.DividerDecoration;
import hikapro.com.backpack.presenter.adapters.helper.items.ItemSwipeCallback;
import hikapro.com.backpack.presenter.adapters.helper.items.StickyHeaderDecoration;

/**
 * Created by tariel on 13/05/16.
 */
public class PackedListPresenter extends ItemListPresenter {

    private TextView backToList;
    private Button unpackButton;


    public PackedListPresenter() {
        this.adapter = new PackedListAdapter(this);
        adapter.setHasStableIds(true);
        adapter.setCheckAll(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        if (savedInstanceState != null)
            set = (Set) savedInstanceState.getSerializable(BUNDLE_SET_LIST_KEY);
        else
            set = getView().getSet();
        android.view.View view = inflater.inflate(R.layout.fragment_packed_list, container, false);
        LinearLayoutManager llm = new LinearLayoutManager(getActivityContext());
        recycler = (RecyclerView) view.findViewById(R.id.items_recycler);
        recycler.setLayoutManager(llm);
        recycler.setAdapter(adapter);

        final StickyHeaderDecoration decoration = new StickyHeaderDecoration(adapter);
        recycler.addItemDecoration(decoration);
        recycler.addItemDecoration(new DividerDecoration(getActivityContext()));
        adapter.registerAdapterDataObserver(new RecyclerView.AdapterDataObserver() {
            @Override
            public void onChanged() {
                decoration.invalidateHeaders();
            }
        });
        ItemSwipeCallback itemSwipeCallback = new ItemSwipeCallback(0, ItemTouchHelper.LEFT, adapter, getActivityContext());

        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(itemSwipeCallback);
        itemTouchHelper.attachToRecyclerView(recycler);
        adapter.setUndoOn(true);

        model.executeQuery();

        backToList      = (TextView) view.findViewById(R.id.back_to_list);
        packedCount     = (TextView) view.findViewById(R.id.header_packed_count);
        unpackButton    = (Button) view.findViewById(R.id.unpack_button);

        if (model.getPackedQty() == model.getActiveItemsCount()) {
            backToList.setVisibility(View.INVISIBLE);
            packedCount.setVisibility(View.INVISIBLE);
            unpackButton.setVisibility(View.VISIBLE);
        } else {
            backToList.setVisibility(View.VISIBLE);
            packedCount.setVisibility(View.VISIBLE);
            unpackButton.setVisibility(View.INVISIBLE);

        }

        unpackButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                model.unpackSet(set.getId());
            }
        });

        backToList.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Fragment fragment = (Fragment) getView();
                if (fragment != null)
                    fragment.getFragmentManager().popBackStack();
            }
        });

        /*
        String str = String.format("%s %d / %d",
                getActivityContext().getResources().getString(R.string.packed),
                model.getPackedQty(), model.getActiveItemsCount());
        packedCount.setText(str);
        */
        return view;
    }


    @Override
    public void setVisibility() {
        if (model.getPackedQty() == 0) {
            packedCount.setVisibility(View.GONE);
            unpackButton.setVisibility(View.GONE);
            backToList.setVisibility(View.VISIBLE);
        } else if (model.getPackedQty() == model.getActiveItemsCount()) {
            packedCount.setVisibility(View.GONE);
            unpackButton.setVisibility(View.VISIBLE);
            backToList.setVisibility(View.GONE);
        } else {
            packedCount.setVisibility(View.VISIBLE);
            unpackButton.setVisibility(View.GONE);
            backToList.setVisibility(View.VISIBLE);
        }
    }
}
