package hikapro.com.backpack.presenter.adapters;

import android.graphics.Color;
import android.os.Handler;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.HashMap;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.entities.Category;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.presenter.Presenter;
import hikapro.com.backpack.presenter.adapters.helper.items.StickyHeaderAdapter;
import hikapro.com.backpack.view.recycler.HeaderViewHolder;
import hikapro.com.backpack.view.recycler.ItemViewHolder;

/**
 * Created by tariel on 01/05/16.
 */
public class ItemListAdapter extends RecyclerView.Adapter<ItemViewHolder> implements StickyHeaderAdapter<HeaderViewHolder> {

    private static final int PENDING_REMOVAL_TIMEOUT = 4000; // 4sec

    boolean undoOn; // is undo on, you can turn it on from the toolbar menu
    private Handler handler = new Handler(); // hanlder for running delayed runnables
    HashMap<Item, Runnable> pendingRunables = new HashMap<>(); // map of items to pending runnables, so we can cancel a removal if need be
    private Presenter.ItemList presenter;
    private boolean checkAll;

    public ItemListAdapter(Presenter.ItemList presenter) {
        this.presenter = presenter;
    }

    @Override
    public int getItemCount() {
        int res = presenter.getModel().getItemsCount();
        return res;
    }

    @Override
    public long getItemId(int position) {
        long ret =  presenter.getModel().getItemId(position);
        return ret;
    }

    @Override
    public void onBindViewHolder(final ItemViewHolder holder, final int position) {

        final Item item = presenter.getModel().getItemByPosition(position);
        if (presenter.getModel().isPendingRemoval(item)) {
            // we need to show the "undo" state of the row
            holder.itemView.setBackgroundColor(Color.RED);
            holder.checkBox.setVisibility(View.GONE);
            holder.undoButton.setVisibility(View.VISIBLE);
            holder.undoButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    // user wants to undo the removal, let's cancel the pending task
                    Runnable pendingRemovalRunnable = pendingRunables.get(item);
                    pendingRunables.remove(item);
                    if (pendingRemovalRunnable != null) handler.removeCallbacks(pendingRemovalRunnable);
                    presenter.getModel().pendingRemoveCancel(item);
                    // this will rebind the row in "normal" state
                    notifyItemChanged(position);
                }
            });
        } else {
            holder.checkBox.setVisibility(View.VISIBLE);
            holder.checkBox.setChecked(checkAll);
            holder.checkBox.setText(item.getName() + " " + item.getId() + " pos " + position);//TODO del
            holder.id = item.getId();
            holder.checkBox.setOnClickListener(new android.view.View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (holder.checkBox.isChecked()) {
                        pack(holder.id, position);
                        // presenter.packItem(); TODO wtf
                    } else {
                        unpack(holder.id, position);
                        // presenter.unpackItem(); TODO wtf
                    }
                }
            });
            holder.infoButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    presenter.showDetails(holder.id);
                }
            });
            holder.itemView.setBackgroundColor(0x33FF99);
            holder.undoButton.setVisibility(View.GONE);
            holder.undoButton.setOnClickListener(null);
        }

    }

    @Override
    public ItemViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        ItemViewHolder viewHolder;
        View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.view_item,
                parent, false);
        viewHolder = new ItemViewHolder(v);
        return viewHolder;
    }

    public void setUndoOn(boolean undoOn) {
        this.undoOn = undoOn;
    }

    public boolean isUndoOn() {
        return undoOn;
    }

    public void add(Item item) {
        presenter.getModel().insertItem(item);
        notifyDataSetChanged();
    }


    public void clear() {
        presenter.getModel().clear();
        notifyDataSetChanged();
    }

    public void filter(String query) {
        presenter.getModel().filter(query);
        notifyDataSetChanged();
    }

    @Override
    public long getHeaderId(int position) {
        /*if (position == 0) {
            return -1;
        } else {*/
            return presenter.getModel().getHeaderId(position);
        //}
    }

    @Override
    public HeaderViewHolder onCreateHeaderViewHolder(ViewGroup parent) {
        HeaderViewHolder viewHolder;
        View v = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.view_header, parent, false);
        viewHolder = new HeaderViewHolder(v);
        return viewHolder;
    }

    @Override
    public void onBindHeaderViewHolder(HeaderViewHolder holder, int position) {
        Category category = presenter.getModel().getCategoryByPosition(position);
        holder.id = category.getId();
        holder.title.setText(category.getName());
        holder.title.setBackgroundColor(0x2B1E15);
    }

    public void pendingRemoval(final int position) {
        final Item item = presenter.getModel().getItemByPosition(position);

        if (! presenter.getModel().isPendingRemoval(item)) {
            presenter.getModel().pendingRemove(item);
            // this will redraw row in "undo" state
            notifyItemChanged(position);
            // let's create, store and post a runnable to remove the item
            Runnable pendingRemovalRunnable = new Runnable() {
                @Override
                public void run() {
                    remove(item, position);
                }
            };
            handler.postDelayed(pendingRemovalRunnable, PENDING_REMOVAL_TIMEOUT);
            pendingRunables.put(item, pendingRemovalRunnable);
        }
    }

    public void remove(Item item, int position) {
        presenter.getModel().deleteItem(item);
        notifyItemRemoved(position);
    }

    public void remove(int position) {
        Item item = presenter.getModel().getItemByPosition(position);
        presenter.getModel().deleteItem(item);
        notifyItemRemoved(position);
    }

    public boolean isPendingRemoval(int position) {
        Item item = presenter.getModel().getItemByPosition(position);
        return presenter.getModel().isPendingRemoval(item);
    }

    public void pack(int itemId, int position) {
        presenter.getModel().packItem(itemId);
        notifyDataSetChanged();
        /*
        notifyItemRemoved(position);
        notifyItemRangeRemoved(position, getItemCount());
        */
    }

    public void unpack(int itemId, int position) {
        presenter.getModel().unpackItem(itemId);
        //notifyItemRemoved(position);
        notifyDataSetChanged();
    }

    public void unpackAll() {

    }

    public void setCheckAll(boolean checkAll) {
        this.checkAll = checkAll;
    }
}
